import {
  selectAll,
  scaleLinear,
  scaleBand,
  dispatch,
  min,
  max,
  axisBottom,
  axisLeft,
  transition,
  format
} from 'd3';
import {
  mouseover,
  mousemove,
  mouseleave,
} from './tooltip';

export const barplot = () => {
  // Register Listener
  const listeners = dispatch(
    'mouseover',
    'mousemove',
    'mouseleave'
  );

  // Dimensions
  let width = 960;
  let height = 500;
  let data = [];
  let margin = {
    top: 0,
    right: 0,
    bottom: 0,
    left: 0,
  };
  let xValue = [];
  let xLabel = '';
  let yValue = [];
  let yLabel = '';
  let chartTitle = '';
  let labelSize = '16px';
  let titleSize = '22px';
  let colors = [
    '#37A2EB',
    '#FF6384',
    '#4CC0C0',
    '#FF9E40',
    '#9A66FF',
    '#FFCD56',
    '#C9CBCF',
  ];
  let tooltip = [];

  // Main function
  const my = (selection) => {
    // Transition
    const t = transition().duration(800);

    // Title
    selection
      .append('text')
      .attr('x', width / 2)
      .attr('y', margin.top + 30)
      .attr('text-anchor', 'middle')
      .style('font-size', titleSize)
      .style('font-family', 'Roboto Condensed')
      .style('font-weight', 'bold')
      .text(chartTitle);

    // X-scale
    const xScale = scaleBand()
      .domain(data.map(xValue))
      .padding(0.2)
      .range([margin.left, width - margin.right]);

    // Draw X-axis
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', '.x-axis')
      .attr(
        'transform',
        `translate(0, ${height - margin.bottom})`
      )
      .call(axisBottom(xScale));

    // Add X axis label:
    selection
      .append('text')
      .attr('text-anchor', 'end')
      .attr('x', width - margin.right)
      .attr('y', height - margin.bottom + 40)
      .style('font-size', labelSize)
      .style('font-family', 'Roboto Condensed')
      .text(xLabel);

    // Y-scale
    const yScale = scaleLinear()
      .domain([0, max(data, yValue) + 1000])
      .range([
        height - margin.bottom,
        margin.top,
      ]);

    // Draw Y-axis
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', '.y-axis')
      .attr(
        'transform',
        `translate(${margin.left}, 0)`
      )
      .call(axisLeft(yScale));

    // Y axis label:
    selection
      .append('text')
      .attr('text-anchor', 'end')
      .attr('transform', 'rotate(-90)')
      .attr('y', margin.left - 50)
      .attr('x', margin.top - 50)
      .style('font-size', labelSize)
      .style('font-family', 'Roboto Condensed')
      .text(yLabel);

    // Position of data-points on canvas
    const marks = data.map((d) => ({
      x: xScale(xValue(d)),
      y: yScale(yValue(d)),
      value: {
       	x: xValue(d),
        y: format(',')(yValue(d)),
      }
    }));

    // Draw Bars
    const bars = selection
      .selectAll('rect')
      .data(marks)
      .join(
        (enter) =>
          enter
            .append('rect')
            .attr('x', (d) => d.x)
            .attr('width', (d) =>
              xScale.bandwidth()
            )
            .attr('y', yScale(0))
            .attr('height', 0)
            .attr('fill', colors[0])
            .on('mouseover', (e, d) => {
              mouseover(e, d.value);
            })
            .on("mousemove", mousemove)
            .on("mouseleave", mouseleave)
            .call((enter) =>
              enter
                .transition(t)
                .attr('y', (d) => d.y)
                .attr(
                  'height',
                  (d) =>
                    height - d.y - margin.bottom
                )
                .delay((d, i) => i * 100)
            ),
        (update) => {},
        (exit) => {}
      );
  };

  // Getters & Setters
  my.width = function (_) {
    return arguments.length
      ? ((width = +_), my)
      : width;
  };
  my.height = function (_) {
    return arguments.length
      ? ((height = +_), my)
      : height;
  };
  my.data = function (_) {
    return arguments.length
      ? ((data = _), my)
      : data;
  };
  my.xValue = function (_) {
    return arguments.length
      ? ((xValue = _), my)
      : xValue;
  };
  my.xLabel = function (_) {
    return arguments.length
      ? ((xLabel = _), my)
      : xLabel;
  };
  my.yValue = function (_) {
    return arguments.length
      ? ((yValue = _), my)
      : yValue;
  };
  my.yLabel = function (_) {
    return arguments.length
      ? ((yLabel = _), my)
      : yLabel;
  };
  my.chartTitle = function (_) {
    return arguments.length
      ? ((chartTitle = _), my)
      : chartTitle;
  };
  my.colors = function (_) {
    return arguments.length
      ? ((colors = _), my)
      : colors;
  };
  my.margin = function (_) {
    return arguments.length
      ? ((margin = _), my)
      : margin;
  };
  my.tooltip = function (_) {
    return arguments.length
      ? ((tooltip = _), my)
      : tooltip;
  };
  // my.on = function (_) {
  //   let value = listeners.on.apply(listeners, arguments);
  //   return value === listeners ? my : value;
  // };

  // Event Listeners

  //! IMP
  return my;
};
